<?php
/**
 * Plugin Name: Freepay.IR - WooCommerce
 * Description: پلاگین پرداخت،شبکه پرداخت فری پی برای WooCommerce
 * Plugin URI: https://freepay.ir
 * Author: تیم فنی فری پی
 * Author URI: http://freepay.ir
 * Version: 1.1
 **/

add_action( 'plugins_loaded', 'woocommerce_freepay_init', 0 );

function woocommerce_freepay_init() {
	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}
	if ( isset( $_GET['msg'] ) ) {
		add_action( 'the_content', 'showMessagefreepay' );
	}

	function showMessagefreepay( $content ) {
		return '<div class="box ' . htmlentities( $_GET['type'] ) . '-box">' . base64_decode( $_GET['msg'] ) . '</div>' . $content;
	}

	class WC_freepay extends WC_Payment_Gateway {
		protected $msg = array();

		public function __construct() {
			$this->id           = 'freepay';
			$this->method_title = __( 'شبکه پرداخت فری پی', 'freepay' );
			$this->has_fields   = false;
			$this->init_form_fields();
			$this->init_settings();
			$this->title            = $this->settings['title'];
			$this->description      = $this->settings['description'];
			$this->merchant_id      = $this->settings['merchant_id'];
			$this->vahed            = $this->settings['vahed'];
			$this->redirect_page_id = $this->settings['redirect_page_id'];
			$this->msg['message']   = "";
			$this->msg['class']     = "";
			add_action( 'woocommerce_api_wc_freepay', array( $this, 'check_freepay_response' ) );
			add_action( 'valid-freepay-request', array( $this, 'successful_request' ) );
			if ( version_compare( WOOCOMMERCE_VERSION, '2.0.0', '>=' ) ) {
				add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
					$this,
					'process_admin_options'
				) );
			} else {
				add_action( 'woocommerce_update_options_payment_gateways', array( $this, 'process_admin_options' ) );
			}
			add_action( 'woocommerce_receipt_freepay', array( $this, 'receipt_page' ) );
		}

		public function init_form_fields() {
			$this->form_fields = array(
				'enabled'          => array(
					'title'   => __( 'فعال سازی/غیر فعال سازی', 'freepay' ),
					'type'    => 'checkbox',
					'label'   => __( 'فعال کردن شبکه پرداخت فری پی', 'freepay' ),
					'default' => 'yes'
				),
				'title'            => array(
					'title'       => __( 'عنوان:', 'freepay' ),
					'type'        => 'text',
					'description' => __( 'عنوانی که کاربر مشاهده خواهد کرد', 'freepay' ),
					'default'     => __( 'شبکه پرداخت فری پی', 'freepay' )
				),
				'description'      => array(
					'title'       => __( 'توضیحات:', 'freepay' ),
					'type'        => 'textarea',
					'description' => __( 'توضیحاتی در مورد درگاه پرداخت / کاربر این توضحیات را مشاهده خواهد کرد', 'freepay' ),
					'default'     => __( '', 'freepay' )
				),
				'merchant_id'      => array(
					'title'       => __( 'کد اعتبارسنجی', 'freepay' ),
					'type'        => 'text',
					'description' => __( 'کد اعتبارسنجی دریافت شده از فری پی', 'freepay' )
				),
				'vahed'            => array(
					'title'       => __( 'واحد پولی' ),
					'type'        => 'select',
					'options'     => array(
						'rial'  => 'ریال',
						'toman' => 'تومان',
					),
					'description' => "نیازمند افزونه ریال و تومان هست"
				),
				'redirect_page_id' => array(
					'title'       => __( 'صفحه بازگشت' ),
					'type'        => 'select',
					'options'     => $this->get_pages( '' ),
					'description' => "ادرس بازگشت بعد از پرداخت"
				)
			);
		}

		public function admin_options() {
			echo '<h3>' . __( 'شبکه پرداخت فری پی', 'freepay' ) . '</h3>';
			echo '<p>' . __( 'شبکه پرداخت فری پی' ) . '</p>';
			echo '<table class="form-table">';
			$this->generate_settings_html();
			echo '</table>';
		}

		public function payment_fields() {
			if ( $this->description ) {
				echo wpautop( wptexturize( $this->description ) );
			}
		}

		public function receipt_page( $order ) {
			echo '<p>' . __( 'Connection To Payment Terminal', 'freepay' ) . '</p>';
			echo $this->generate_freepay_form( $order );
		}

		public function process_payment( $order_id ) {
			$order = new WC_Order( $order_id );

			return array( 'result' => 'success', 'redirect' => $order->get_checkout_payment_url( true ) );
		}

		public function check_freepay_response() {
			global $woocommerce;
			$order_id = $woocommerce->session->zegersot;
			$order    = new WC_Order( $order_id );
			if ( $order_id != '' ) {
				if ( $order->status != 'completed' ) {
					$merchant_id = $this->merchant_id;
					$authority = (int) trim($_GET['authority']);

					$ch = curl_init();
					curl_setopt( $ch, CURLOPT_URL, 'http://freepay.ir/payment/verify' );
					curl_setopt( $ch, CURLOPT_POSTFIELDS, "authority=$authority&api=$merchant_id" );
					curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
					curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
					$response = curl_exec( $ch );
					curl_close( $ch );

					$result = json_decode( $response );
					if ( $result->status == 1 && $result->message == 'ok' && $woocommerce->session->zegersot_id == $authority ) {
						$this->msg['message'] = 'Successful Payment<br/>Reference ID : ' . $authority;
						$this->msg['class']   = 'success';
						$order->payment_complete();
						$order->add_order_note( 'Successful Payment<br/>Reference ID : ' . $authority . ' AND ' . $authority );
						$order->add_order_note( $this->msg['message'] );
						$woocommerce->cart->empty_cart();
					} else {
						$this->msg['class']   = 'error';
						$this->msg['message'] = "Payment failed";
					}
				} else {
					$this->msg['class']   = 'error';
					$this->msg['message'] = 'We can not find order information';
				}
			}
			$redirect_url = ( $this->redirect_page_id == "" || $this->redirect_page_id == 0 ) ? get_site_url() . "/" : get_permalink( $this->redirect_page_id );
			$redirect_url = add_query_arg( array(
				'msg'  => base64_encode( $this->msg['message'] ),
				'type' => $this->msg['class']
			), $redirect_url );
			wp_redirect( $redirect_url );
			exit();
		}

		public function showMessage( $content ) {
			return '<div class="box ' . $this->msg['class'] . '-box">' . $this->msg['message'] . '</div>' . $content;
		}

		public function generate_freepay_form( $order_id ) {
			global $woocommerce;
			$order        = new WC_Order( $order_id );
			$redirect_url = ( $this->redirect_page_id == "" || $this->redirect_page_id == 0 ) ? get_site_url() . "/" : get_permalink( $this->redirect_page_id );
			$redirect_url = add_query_arg( 'wc-api', get_class( $this ), $redirect_url );
			unset( $woocommerce->session->zegersot );
			unset( $woocommerce->session->zegersot_id );
			$woocommerce->session->zegersot = $order_id;
			$amount                         = $order->order_total;

			$merchant_id  = $this->merchant_id;
			$amount       = ( $this->vahed == 'toman' ) ? $amount * 10 : $amount;
			$callback_url = urlencode($redirect_url);
			$host=$_SERVER['SERVER_NAME'];
			$ch = curl_init();
			curl_setopt( $ch, CURLOPT_URL, 'http://freepay.ir/payment/request' );
			curl_setopt( $ch, CURLOPT_POSTFIELDS, "api=$merchant_id&amount=$amount&callback=$callback_url&host=$host" );
			curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
			$response = curl_exec( $ch );
			curl_close( $ch );

			$result = json_decode( $response );
			if ( $result->type == 'success' ) {
				$woocommerce->session->zegersot_id = $result->auth;
				$redirect_page = "https://freepay.ir/payment/gateway:".$result->auth;
				if ( ! headers_sent() ) {
					header( 'Location: ' . $redirect_page );
					exit();
				} else {
					echo "<script type='text/javascript'>window.onload = function () { top.location.href = '" . $redirect_page . "'; };</script>";
					exit();
				}
			} else {
				echo $result->message;
			}
		}

		public function get_pages( $title = false, $indent = true ) {
			$wp_pages  = get_pages( 'sort_column=menu_order' );
			$page_list = array();
			if ( $title ) {
				$page_list[] = $title;
			}
			foreach ( $wp_pages as $page ) {
				$prefix = '';
				if ( $indent ) {
					$has_parent = $page->post_parent;
					while ( $has_parent ) {
						$prefix     .= ' - ';
						$next_page  = get_page( $has_parent );
						$has_parent = $next_page->post_parent;
					}
				}
				$page_list[ $page->ID ] = $prefix . $page->post_title;
			}

			return $page_list;
		}
	}

	function woocommerce_add_freepay_gateway( $methods ) {
		$methods[] = 'WC_freepay';

		return $methods;
	}

	add_filter( 'woocommerce_payment_gateways', 'woocommerce_add_freepay_gateway' );
}